/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section;

import java.util.ArrayList;
import java.util.Collection;
import java.util.EnumSet;

import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.permission.CRUD;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CacheRelevance;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICacheRelevantFormElement;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectableElement;
import eu.etaxonomy.taxeditor.ui.element.ToggleableTextElement;

/**
 * This class visualizes an CDM entity of type ENTITY.
 *
 * @param <ENTITY> A CDM entity which should be visualized by this section.
 *
 * @author n.hoffmann
 * @created Feb 26, 2010
 */
public abstract class AbstractCdmDetailSection<ENTITY>
			extends AbstractFormSection<ENTITY>
			implements IEnableableFormElement, IExpansionListener, ICacheRelevantFormElement {

	protected AbstractCdmDetailElement<ENTITY> detailElement;

	public AbstractCdmDetailSection(CdmFormFactory formFactory,
			ConversationHolder conversation, ICdmFormElement parentElement,
			ISelectionProvider selectionProvider, int style) {
	    this(formFactory, null, conversation, parentElement, selectionProvider, style);
	}

	//TODO remove conversation
	public AbstractCdmDetailSection(CdmFormFactory formFactory, Class<ENTITY> clazz,
			ConversationHolder conversation, ICdmFormElement parentElement,
			ISelectionProvider selectionProvider, int style) {
		super(formFactory, parentElement, selectionProvider,
				ExpandableComposite.CLIENT_INDENT | style);

        setText(getHeading());

        addExpansionListener(this);

        if(clazz==null){
            createControls(this, style);
        }
        else{
            createControlsByType(this, clazz, SWT.NULL);
        }
	}

	@Override
	public void setText(String title) {
	    super.setText(StoreUtil.cleanTitleString(title));
	}

	public AbstractCdmDetailElement<ENTITY> getDetailElement() {
        return detailElement;
    }

    public void setDetailElement(AbstractCdmDetailElement<ENTITY> detailElement) {
        this.detailElement = detailElement;
    }

    protected void createControlsByType(AbstractCdmDetailSection<ENTITY> formElement, Class<ENTITY> entityClass, int style) {
	    TableWrapLayout layout = (TableWrapLayout) getLayoutComposite().getLayout();
	    layout.topMargin = 10;
	    layout.numColumns = DEFAULT_NUM_COLUMNS;

	    getLayoutComposite().setLayout(layout);
	    if(entityClass==null){
	        detailElement = createCdmDetailElement(formElement, style);
	    }
	    else{
	        detailElement = createCdmDetailElementByType(formElement, entityClass, style);
	    }
	}

	protected void createControls(AbstractCdmDetailSection<ENTITY> formElement, int style) {
	    createControlsByType(formElement, null, style);
	}

	protected abstract AbstractCdmDetailElement<ENTITY> createCdmDetailElement(AbstractCdmDetailSection<ENTITY> parentElement, int style);

	protected AbstractCdmDetailElement<ENTITY> createCdmDetailElementByType(AbstractCdmDetailSection<ENTITY> parentElement, Class<ENTITY> entityClass, int style){
	    return createCdmDetailElement(parentElement, style);
	}

	public abstract String getHeading();

	@Override
	public void dispose() {
		if (detailElement instanceof ISelectableElement) {
			ISelectableElement selectableElement = (ISelectableElement) detailElement;
			if (selectableElement.getSelectionArbitrator() != null) {
				formFactory.destroySelectionArbitrator(selectableElement
						.getSelectionArbitrator());
			}
		}
		super.dispose();
	}

	@Override
	public void setBackground(Color color) {
		if (detailElement != null) {
		    detailElement.setBackground(color);
		}
		super.setBackground(color);
	}

	@Override
	public void setEntity(ENTITY entity) {
	    EnumSet<CRUD> requiredCrud;
	    if(entity == null || (entity instanceof CdmBase &&StoreUtil.getCdmEntity(entity).getId() == 0 )) {
            // new entity, not yet saved
            requiredCrud = EnumSet.of(CRUD.CREATE);
        } else {
            requiredCrud = EnumSet.of(CRUD.UPDATE);
        }
		if (detailElement != null) {
			detailElement.setEntity(entity);
			formFactory.adapt(detailElement);
	        Collection<Object> except = new ArrayList<>();
	        for(ICdmFormElement formElement:detailElement.getElements()){
	            if(formElement instanceof IEnableableFormElement && !((IEnableableFormElement) formElement).isEnabled()){
	                except.add(formElement);
	            }
	        }
	        if (entity != null && entity instanceof CdmBase){
	            boolean hasPermission = CdmStore.currentAuthentiationHasPermission(StoreUtil.getCdmEntity(entity), requiredCrud);
	            detailElement.setEnabled(isEnabled() && hasPermission, except);
	        }else{
	            detailElement.setEnabled(isEnabled() && true, except);
	        }

		}
		super.setEntity(entity);
		setSectionTitle();
		layout();
	}

	protected void setSectionTitle() {
		String title = "";
		String label = "";
		if (getEntity() != null && (getEntity() instanceof IdentifiableEntity) && !(getEntity() instanceof SpecimenOrObservationBase)) {
			if (getEntity() instanceof DefinedTermBase<?> ){
				label = ((DefinedTermBase<?>)getEntity()).getPreferredRepresentation(PreferencesUtil.getGlobalLanguage()).getLabel();
//				if (label == null){
//					label = ((DefinedTermBase<?>)getEntity()).getLabel();
//				}
			} else if (getEntity() instanceof TextData){
				label = ((TextData)getEntity()).getLanguageText(PreferencesUtil.getGlobalLanguage()).getText();
				if (label == null){
					label = ((TextData)getEntity()).getLanguageText(Language.DEFAULT()).getText();
				}
			}
			if(CdmUtils.isBlank(label)){
			    label = ((IdentifiableEntity<?>) getEntity()).getTitleCache();
			}
			if(CdmUtils.isBlank(label)){
			    label = ((IdentifiableEntity<?>) getEntity()).generateTitle();
			}
			title = ": " + label;
		}
		this.setText(String.format("%s%s", getHeading(), title));
		setTextClient(createToolbar());
	}

	protected Control createToolbar() {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);
		return toolBarManager.createControl(this);
	}

	public void updateTitle() {
		if (!isDisposed()) {
			setSectionTitle();
			layout();
		}
	}

    @Override
    public void updateCacheRelevance() {
        if (detailElement != null) {
            detailElement.updateCacheRelevance();
        }
    }

    @Override
    public void addDependsOnCache(ToggleableTextElement toggleElement) {
        if (detailElement != null) {
            detailElement.addDependsOnCache(toggleElement);
        }
    }

    @Override
    public CacheRelevance cacheRelevance() {
        return detailElement.cacheRelevance();
    }

    @Override
    public void expansionStateChanging(ExpansionEvent e) {
		// logger.warn("Expansion State Changing");
	}

	@Override
    public void expansionStateChanged(ExpansionEvent e) {
		// logger.warn("Expansion State Changed");
	}

	public void setIndent(int indent){
	    TableWrapData tableWrapData = (TableWrapData)this.getLayoutData();
	    if (tableWrapData == null){
	        tableWrapData = new TableWrapData();
	    }
	    tableWrapData.indent = indent;
	    setLayoutData(tableWrapData);
	}
}
