/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.ui.section;

import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.AbstractFormSection;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEntityElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectable;
import eu.etaxonomy.taxeditor.ui.element.LabelElement;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * Visualizes an element of type ENTITY in an {@link AbstractEntityCollectionSection}
 * and links listener functionalities to it.
 *
 * @param ENTITY the type of the element which is visualized by this class
 *
 * @author n.hoffmann
 * @created Nov 16, 2009
 */
public abstract class AbstractEntityCollectionElement<ENTITY>
        extends AbstractCdmFormElement
        implements IEntityElement<ENTITY>, SelectionListener,
              IConversationEnabled, IEnableableFormElement {

	protected ENTITY entity;

	private final Composite container;

	/**
	 * Composite "around" the actual content. Is used for control action like e.g. remove button
	 */
	private final Composite box;

	private boolean isEnabled;

	private Button btnRemove;

	private Button btnChooseEntity;

	private Color backgroundColor;

	private LabelElement warnForReferencedObjects;

	public AbstractEntityCollectionElement(CdmFormFactory formFactory,
			AbstractFormSection section, ENTITY entity, SelectionListener removeListener,
			Color backgroundColor, int style) {
		super(formFactory, (ICdmFormElement) section);
		this.entity = entity;

		init();

		formFactory.addPropertyChangeListener(this);

		box = formFactory.createComposite(section.getLayoutComposite());
		box.setBackgroundMode(SWT.INHERIT_DEFAULT);
		addControl(box);

		TableWrapLayout boxLayout = LayoutConstants.LAYOUT(4, false);
		boxLayout.topMargin = 4;
		boxLayout.bottomMargin = 4;
		box.setLayout(boxLayout);

		box.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		container = formFactory.createComposite(box);
		container.setBackgroundMode(SWT.INHERIT_DEFAULT);

		setLayoutComposite(container);

		addControl(container);
		TableWrapLayout containerLayout = LayoutConstants.LAYOUT(2, false);
		containerLayout.horizontalSpacing = 5;

		container.setLayout(containerLayout);
		container.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());

		if (removeListener != null) {
			btnRemove = formFactory.createButton(box, null, SWT.PUSH);
			addControl(btnRemove);
			btnRemove.setLayoutData(LayoutConstants.RIGHT());
			btnRemove.setImage(ImageResources
					.getImage(ImageResources.TRASH_ICON));
			btnRemove.setToolTipText("Remove");

			btnRemove.addSelectionListener(removeListener);
			btnRemove.setEnabled(isEnabled);
		}



		setEntity(entity);
		createControls(this, style);
	}

	/**
	 * Init gets executed before any other setup of the section takes place
	 *
	 * Implement this if you want to configure the section
	 */
	public void init() {
		// default implementation is empty
	}

	public abstract void setEntity(ENTITY entity);

	@Override
    public ENTITY getEntity() {
		return entity;
	}

	/**
	 * Sub classes should override to provide the functionality to choose the
	 * entity from existing ones from the data source.<br>
	 * <b>Note:</b> to enable this functionality sub classes have to set
	 * the corresponding flag in the super constructor
	 * @return an existing entity from the data source
	 */
	protected ENTITY selectFromDialog(){
	    return null;
	}

	public abstract void createControls(ICdmFormElement element, int style);

	/**
	 * Mark <code>this</code> element as selected.
	 */
	@Override
    public void setSelected(boolean selected) {

		for (ICdmFormElement element : getElements()) {
			if (element instanceof ISelectable) {
				((ISelectable) element).setSelected(selected);
			}
		}
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {
		if (event == null) {
			return;
		}
		Object eventSource = event.getSource();
		if (getElements().contains(eventSource) || getControls().contains(eventSource)) {
			handleEvent(eventSource);
		}
	}

	public abstract void handleEvent(Object eventSource);

	@Override
    public void setBackground(Color color) {
	    if(box.isDisposed() || container.isDisposed()){
	        return;
	    }
		backgroundColor = color;
		super.setBackground(color);
		box.setBackground(color);
		container.setBackground(color);
	}

	/**
	 * {@inheritDoc}
	 *
	 * React when selection occurs
	 */
	@Override
    public void widgetSelected(SelectionEvent e) {

	}

	@Override
    public void widgetDefaultSelected(SelectionEvent e) {
	}

	@Override
	public Composite getLayoutComposite() {
		return container;
	}

	public Color getBackgroundColor() {
		return backgroundColor;
	}

    public Composite getBox() {
        return box;
    }

	@Override
    public ConversationHolder getConversationHolder() {
		if (getParentElement() instanceof IConversationEnabled) {
			return ((IConversationEnabled) getParentElement())
					.getConversationHolder();
		}
		throw new IllegalArgumentException(
				"Parent element should be IConversationEnabled");
	}

	@Override
    public void update(CdmDataChangeMap changeEvents) {
	}

    public void setWarnForReferencedObjects(LabelElement warnForReferencedObjects) {
        this.warnForReferencedObjects = warnForReferencedObjects;
    }

    public void setWarnForReferencingObjects(ICdmFormElement formElement, int defaultReferencingObjects){
        if (entity instanceof CdmBase){
            CdmBase cdmBase = (CdmBase)entity;
            if (cdmBase.getId() != 0){
                long referencingObjectsCount = CdmStore.getCommonService().getReferencingObjectsCount(cdmBase);

                if (referencingObjectsCount > defaultReferencingObjects){
                    setWarnForReferencedObjects(formFactory.createLabel(formElement, CdmUtils.Nz("The "+ cdmBase.getUserFriendlyTypeName()+" is referenced by more than one object, if you change it, it is changed for all these objects")));
                    warnForReferencedObjects.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
                    warnForReferencedObjects.setLayout(LayoutConstants.FILL(2, 3));
                    warnForReferencedObjects.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_RED));
                }
            }
        }
    }

    public void setWarnForReferencingObjects(ICdmFormElement formElement){
        setWarnForReferencingObjects(formElement, 1);
    }

    public void setWarnForReferencingObjectsVisible(boolean isVisible){
        if (warnForReferencedObjects != null){
            warnForReferencedObjects.setVisible(isVisible);
        }
    }

    @Override
	public void setEnabled(boolean enabled) {
		this.isEnabled = enabled;
		if (btnRemove != null){
			btnRemove.setEnabled(enabled);
		}
		if (btnChooseEntity != null){
			btnChooseEntity.setEnabled(enabled);
		}

		for (ICdmFormElement element:this.getElements()){
			if (element instanceof IEnableableFormElement){
				((IEnableableFormElement)element).setEnabled(enabled);
			}
		}
	}

	@Override
	public boolean isEnabled() {
		return isEnabled;
	}
}
