/**
* Copyright (C) 2019 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.ui.section.feature;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.api.service.ITermNodeService;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.FeatureState;
import eu.etaxonomy.cdm.model.description.State;
import eu.etaxonomy.cdm.model.term.DefinedTermBase;
import eu.etaxonomy.cdm.model.term.TermNode;
import eu.etaxonomy.cdm.model.term.TermVocabulary;
import eu.etaxonomy.cdm.persistence.dto.FeatureDto;
import eu.etaxonomy.cdm.persistence.dto.FeatureStateDto;
import eu.etaxonomy.cdm.persistence.dto.TermDto;
import eu.etaxonomy.cdm.persistence.dto.TermNodeDto;
import eu.etaxonomy.cdm.persistence.dto.TermVocabularyDto;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.termtree.AvailableTermsWizard;
import eu.etaxonomy.taxeditor.ui.AbstractEntityCollectionElementWizardPage;
import eu.etaxonomy.taxeditor.ui.combo.term.TermComboElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * @author pplitzner
 * @since Aug 16, 2019
 *
 */
public class FeatureStateWizardPage extends AbstractEntityCollectionElementWizardPage {

    private TermComboElement<Feature> comboFeature;
    private Button btnSelectState;

    private TermNodeDto termNode;
    private State selectedState = null;
    private List<TermVocabularyDto> stateVocabulariesDtos = new ArrayList<>();
    private Text text;

    protected FeatureStateWizardPage(String pageName, TermNodeDto termNode, CdmFormFactory formFactory) {
        super(pageName, formFactory);
        this.termNode = termNode;
        setTitle("Create Feature State");
        setDescription("Select Feature and State");
    }

    @Override
    public void createControl(Composite parent) {
        super.createControl(parent);

        comboFeature = formFactory.createDefinedTermComboElement(Collections.EMPTY_LIST, rootElement, "Parent Feature", null, SWT.NONE);

        Label label = new Label(rootElement.getLayoutComposite(), SWT.NONE);
        label.setText("State");
        Composite buttonTextComposite = new Composite(rootElement.getLayoutComposite(), SWT.NONE);
        buttonTextComposite.setLayoutData(LayoutConstants.FILL(1, 1));
        buttonTextComposite.setLayout(new GridLayout(2, false));
        text = new Text(buttonTextComposite, SWT.NONE);
        text.setEditable(false);
        text.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        btnSelectState = new Button(buttonTextComposite, SWT.PUSH);
        btnSelectState.setImage(ImageResources.getImage(ImageResources.BROWSE_ICON));
        btnSelectState.addSelectionListener(new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                AvailableTermsWizard wizard = new AvailableTermsWizard(stateVocabulariesDtos);
                wizard.setSingleSelection(true);
                WizardDialog dialog = new WizardDialog(getShell(), wizard);

                if (dialog.open() == IStatus.OK) {
                    Collection<DefinedTermBase> additionalTerms = wizard.getAdditionalFeatures();
                    if(additionalTerms.size()==1){
                        selectedState = (State) additionalTerms.iterator().next();
                        text.setText(selectedState.getLabel());
                        FeatureStateWizardPage.this.getWizard().getContainer().updateButtons();
                    }
                }
            }
        });

        formFactory.addPropertyChangeListener(this);

        updateFeatureCombo();
    }

    FeatureStateDto getFeatureStateDto(){
        return new FeatureStateDto(null, FeatureDto.fromFeature(comboFeature.getSelection()), TermDto.fromTerm(selectedState));
    }
    FeatureState getFeatureState(){
        return FeatureState.NewInstance(comboFeature.getSelection(), selectedState);
    }
    private void updateFeatureCombo(){
        List<Feature> features = new ArrayList<>();
        UUID parentUuid = termNode.getParentUuid();
        if (parentUuid != null){
            TermNode<? extends Feature> parent = CdmStore.getService(ITermNodeService.class).load(parentUuid);
            while(parent!=null){
                Feature parentTerm = parent.getTerm();
                if(parentTerm!=null
                        && parentTerm.isSupportsCategoricalData()
                        && !parentTerm.getSupportedCategoricalEnumerations().isEmpty()){
                    features.add(parentTerm);
                }
                parent = parent.getParent();
            }
        }
        comboFeature.setTerms(features);
        if(features.size()==1){
            Feature feature = features.get(0);
            comboFeature.setSelection(feature);
            updateStateCombo(feature);
            comboFeature.removeEmptyElement();
        }
    }

    private void updateStateCombo(Feature feature){
        stateVocabulariesDtos.clear();
        text.setText("");
        selectedState = null;
        if(feature!=null){
            Set<TermVocabulary<State>> stateVocabularies = feature.getSupportedCategoricalEnumerations();
            for (TermVocabulary<State> termVocabulary : stateVocabularies) {
                stateVocabulariesDtos.add(new TermVocabularyDto(termVocabulary.getUuid(), termVocabulary.getRepresentations(), termVocabulary.getTermType(), termVocabulary.getTitleCache(), termVocabulary.isAllowDuplicates(), termVocabulary.isOrderRelevant(), termVocabulary.isFlat()));
            }
        }
    }

    @Override
    public boolean isPageComplete() {
        return comboFeature.getSelection()!=null && selectedState!=null;
    }

    @Override
    public void propertyChange(PropertyChangeEvent event) {
        if(event.getSource()==comboFeature){
            Feature feature = comboFeature.getSelection();
            updateStateCombo(feature);
            comboFeature.removeEmptyElement();
        }
        getWizard().getContainer().updateButtons();
    }

}
