/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.ui.section.name;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.INonViralName;
import eu.etaxonomy.cdm.model.name.TaxonName;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.event.EventUtility;
import eu.etaxonomy.taxeditor.event.WorkbenchEventConstants;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.remoting.CdmEagerLoadingException;
import eu.etaxonomy.taxeditor.ui.dialog.selection.NameSelectionDialog;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailElement;
import eu.etaxonomy.taxeditor.ui.section.AbstractCdmDetailSection;
import eu.etaxonomy.taxeditor.ui.section.ITaxonBaseDetailSection;

/**
 * @author n.hoffmann
 * @created May 20, 2010
 */
public class NonViralNameDetailSection extends AbstractCdmDetailSection<TaxonName>
		implements ITaxonBaseDetailSection {

	private TaxonBase<?> taxonBase;
	boolean nameChoosable = false;

	/**
	 * <p>Constructor for NonViralNameDetailSection.</p>
	 *
	 * @param formFactory a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory} object.
	 * @param conversation a {@link eu.etaxonomy.cdm.api.conversation.ConversationHolder} object.
	 * @param parentElement a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement} object.
	 * @param selectionProvider a {@link org.eclipse.jface.viewers.ISelectionProvider} object.
	 * @param nameChoosable if <code>true</code> adds a button to choose the displayed name
	 * @param style a int.
	 */
	public NonViralNameDetailSection(CdmFormFactory formFactory, ConversationHolder conversation,
			ICdmFormElement parentElement, ISelectionProvider selectionProvider, boolean nameChoosable, int style) {
		super(formFactory, conversation, parentElement, selectionProvider, style);
		this.nameChoosable = nameChoosable;
	}

	@Override
	protected Control createToolbar() {
	        ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);

	        if(nameChoosable ){
	            //choose name
	            Action chooseNameAction = new Action("Choose Name", IAction.AS_PUSH_BUTTON){
	                @Override
	                public void run() {
	                    TaxonName taxonName = NameSelectionDialog.select(getShell(), //getConversationHolder(),
	                            getEntity());
	                    if(taxonName!=null){
                            TaxonName nonViralName = HibernateProxyHelper.deproxy(taxonName);
                            taxonBase.setName(nonViralName);
                            setEntity(nonViralName);

                            firePropertyChangeEvent(NonViralNameDetailSection.this);
                            EventUtility.postEvent(WorkbenchEventConstants.REFRESH_DETAILS, true);
                            EventUtility.postEvent(WorkbenchEventConstants.REFRESH_SUPPLEMENTAL, true);
	                    }
	                }
	            };
	            chooseNameAction.setToolTipText("Choose name for this taxon");
	            chooseNameAction.setImageDescriptor(ImageResources.getImageDescriptor(ImageResources.BROWSE_ICON));

	            toolBarManager.add(chooseNameAction);
	    }

		//clone
		if(getEntity() != null && checkForMultipleNameUsages(getEntity())){

			Action cloneAction = new Action("Clone", IAction.AS_PUSH_BUTTON){
			    @Override
			    public void run() {
					boolean confirm = MessagingUtils.confirmDialog("Confirm cloning", "Do you really want to clone the name?");

					if(confirm){
						TaxonName clonedName;
						clonedName = (TaxonName) getEntity().clone();
						setEntity(clonedName);
						taxonBase.setName(clonedName);
						taxonBase.generateTitle();
						firePropertyChangeEvent(new CdmPropertyChangeEvent(NonViralNameDetailSection.this, null));
					}

				}
			};

			cloneAction.setToolTipText("Clone the name if you do not want to edit the shared instance");
			cloneAction.setImageDescriptor(ImageResources.getImageDescriptor(ImageResources.COPY_ICON));
			toolBarManager.add(cloneAction);

		}
		return toolBarManager.createControl(this);
	}

	private boolean checkForMultipleNameUsages(INonViralName nonViralName) {
	    try{
	        return nonViralName.getTaxonBases().size() != 1;
	    }catch (CdmEagerLoadingException e){
	        System.out.println(e.getLocalizedMessage());
	        return false;
	    }
	}

	@Override
	public String getHeading() {
		return "Name";
	}

	@Override
    public void setTaxonBase(TaxonBase taxon) {
		taxonBase = taxon;
		TaxonName name = HibernateProxyHelper.deproxy(taxon.getName());
		setEntity(name);
	}

	@Override
	public TaxonBase getTaxonBase() {
		return taxonBase;
	}

	@Override
	protected AbstractCdmDetailElement<TaxonName> createCdmDetailElement(
			AbstractCdmDetailSection<TaxonName> parentElement, int style) {
	    return formFactory.createNonViralNameDetailElement(parentElement);
	}
}
